/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 package test.performance;

// used to get attributes// used to get child nodes
// used to get attributes
// used to get child nodes
import java.util.Date;

import org.elvin.je4.ElvinURL;
import org.elvin.je4.Notification;
import org.elvin.je4.Producer;



/**
 * This test sends 1000 events to elvin and calculates the delay of processing them
 * A subscription to these events is assumed to exist in order to force the events
 * to be routed throuth.
 * 
 * @author Roberto Silveira Silva Filho [rsilvafi@ics.uci.edu]
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ElvinThroughputPublishTest  {

	static int repetitions = 10000;
	
	Date beginPublish = null;
	Date endPublish = null;
	
	Date beginEventCreation = null;
	Date endEventCreation = null;
	
	int notifCounter = 0; // number of notifications received

	public static void main(String argv[]) {

		Producer elvinProducer = null;
		ElvinURL url;

		Notification[] eventArray = new Notification[repetitions];	
		String hostname;

		if (argv.length != 1) {
			System.err.println(
				"Usage: java ElvinThroughputTest hostname ");
			System.exit(1);
		}

		hostname = argv[0];
		
		url = new ElvinURL(hostname);
		try {
		elvinProducer = new Producer(url);
		} catch (java.io.IOException ex) {
			System.out.println("Error when initializing producer !");
			System.out.println(ex);
		}
		
		// Create an instance of this class
		ElvinThroughputPublishTest myInstance = null;
				
		try {
			myInstance = new ElvinThroughputPublishTest();	
			
			
			System.out.println("ElvinThroughputTest: Generating "+repetitions+" events...");
			myInstance.beginEventCreation = new Date();
			for (int i=0; i<repetitions; i++) {
				eventArray[i] = myInstance.newEvent(i);
			}
			myInstance.endEventCreation = new Date();
			
			
			System.out.println("ElvinThroughputTest: Publishing "+repetitions+" events...");
			myInstance.beginPublish = new Date();
			for (int i=0; i<repetitions; i++) {
				elvinProducer.notify(eventArray[i]);
			}
			myInstance.endPublish = new Date();
			
			System.out.println("Finalizing...");
			elvinProducer.close();
		
		} catch (java.io.IOException ex) {
			System.out.println(ex);
		}

		myInstance.printStatistics();
		
	} // main

	public Notification newEvent(int counter) {
		Notification event = new Notification();
		event.put("name", "Roberto");
		event.put("age", 29);
		event.put("counter", counter);
		
		return event;
	}

	public void printStatistics() {
		
		System.out.println("\n Performance data: \n");
		
		System.out.print("Total delay to create "+repetitions+" events = ");
		System.out.println(endEventCreation.getTime()-beginEventCreation.getTime()+"ms");
		
		System.out.print("Total delay to publish "+repetitions+" events = ");
		System.out.println(endPublish.getTime()-beginPublish.getTime()+"ms");
				
	}




}