/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package test.performance;

// used to get attributes// used to get child nodes
// used to get attributes
// used to get child nodes
import java.io.IOException;
import java.util.Date;

import org.elvin.je4.Consumer;
import org.elvin.je4.ElvinURL;
import org.elvin.je4.Notification;
import org.elvin.je4.Subscription;

/**
 * This test sends 1000 events to elvin and calculates the delay of processing them
 * A subscription to these events is assumed to exist in order to force the events
 * to be routed throuth.
 * 
 * @author Roberto Silveira Silva Filho [rsilvafi@ics.uci.edu]
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ElvinThroughputSubscriberTest
	implements org.elvin.je4.NotificationListener {

	int notifCounter = 0;
	static int repetitions = 100000;

	Date beginDate;
	Date endDate;

	public static void main(String argv[]) {
		Consumer elvinConsumer = null;
		ElvinURL url;

		String hostname;

		if (argv.length != 1) {
			System.err.println(
				"Usage: java ElvinThroughputSubscriberTest hostname ");
			System.exit(1);
		}

		hostname = argv[0];

		url = new ElvinURL(hostname);

		try {
			elvinConsumer = new Consumer(url);
		} catch (java.io.IOException ex) {
			System.out.println(
				"Error when reading producer and consumer objects!");
			System.out.println(ex);
		}

		// Create an instance of this class
		ElvinThroughputSubscriberTest myInstance = null;

		System.out.println("ElvinThroughputSubscriberTest: Subscribing...");

		myInstance = new ElvinThroughputSubscriberTest();

		System.out.println(
			"ElvinThroughputSubscriberTest: Posting 1 subscription...");

		//Subscription mySubscription =	new Subscription("(name == 'Roberto') && (age >18)");
		Subscription mySubscription = new Subscription("equals(name, 'Roberto')");
		mySubscription.addNotificationListener(myInstance);

		try {
			elvinConsumer.addSubscription(mySubscription);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		System.out.println(
			"ElvinThroughputSubscriberTest: Waiting for "+repetitions+" notifications...");

		Notification event;
		int eventRate = 0;
		int lastCount = 0;
		while (myInstance.notifCounter < repetitions) {

			// Keep the program alive
			try {
				Thread.sleep(1000);
				eventRate = myInstance.notifCounter - lastCount;
				lastCount = myInstance.notifCounter;
				System.out.println("Got "+myInstance.notifCounter+" notifications up to now. "+eventRate+" events/s");
			} catch (InterruptedException ex) {
				System.out.println(ex);
			}
		}

		myInstance.printStatistics();
		
		// wait some time...
		try {
			Thread.sleep(1000);
		} catch (InterruptedException ex) {
			System.out.println(ex);
		}

		System.out.println("Finalizing...");
		elvinConsumer.close();

	}

	public void printStatistics() {
		System.out.println();
		System.out.println("End date: " + endDate);
		System.out.println("Begin date: " + beginDate);
		long difference = endDate.getTime() - beginDate.getTime();
		double rate = repetitions / difference;
		System.out.println("Total time: " + difference + " ms");
		System.out.println("Average throughput: " + rate + " events/ms");
		System.out.println(
			"Average throughput: "
				+ repetitions / (difference / 1000)
				+ " events/s");
	}

	/* (non-Javadoc)
	 * @see org.elvin.je4.NotificationListener#notificationAction(org.elvin.je4.Notification)
	 */
	public void notificationAction(Notification arg0) {
		if (notifCounter == 0)
			beginDate = new Date();

		notifCounter++;

		if (notifCounter >= repetitions)
			endDate = new Date();

		//System.out.println(arg0.printString());

	}

}
