/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 package test.performance;

// used to get attributes// used to get child nodes
// used to get attributes
// used to get child nodes
import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;

import edu.uci.isr.yancees.EventInterface;
import edu.uci.isr.yancees.GenericSubscription;
import edu.uci.isr.yancees.SubscriberInterface;
import edu.uci.isr.yancees.YanceesEvent;
import edu.uci.isr.yancees.YanceesException;
import edu.uci.isr.yancees.client.rmi.YanceesRMIClient;

public class YanceesEventNotificationTest  {

	static int repetitions = 100;
	
	public YanceesEventNotificationTest() throws RemoteException {
		super();
	}

	static long[] beginSubscribe = new long[repetitions];
	static long[] endSubscribe = new long[repetitions];
	static long[] beginPublish = new long[repetitions];
	static long[] endPublish = new long[repetitions];
	static long eventSent;
	static long[] notificationReceived = new long[repetitions];
	static long[] averageCycles = new long[repetitions];
	
	static int notifCounter = 0; // number of notifications received

	public static void main(String argv[]) {
		//RemoteYanceesInterface yanceesRemote = null;
		String hostname;
		String subscriptionFileName;
		

		if (argv.length != 2) {
			System.err.println(
				"Usage: java YanceesEventNotificationTest hosthame subscription.xml");
			System.exit(1);
		}

		hostname = argv[0];
		subscriptionFileName = argv[1];
		

		YanceesSubscriber subscriber = new YanceesSubscriber(hostname, subscriptionFileName);
		subscriber.start();
		
		// give some time for the subscription to happen....
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		YanceesPublisher publisher = new YanceesPublisher(hostname);
		publisher.start();

	} // main
	
	
	public YanceesEvent newEvent(int counter) {
		YanceesEvent event = new YanceesEvent();
		event.put("name", "Roberto");
		event.put("age", 29);
		event.put("counter", counter);
		return event;
	}
	

	public static void printStatistics() {
		long period;
		long sum;
		System.out.println("\n Performance data: \n");
		
		System.out.println("\nNotifications Received: "+notifCounter);
		System.out.println("\nSubscription delays:");
		sum = 0;
		for (int i=0; i<1; i++) {
			period = endSubscribe[i] - beginSubscribe[i];
			//System.out.println("Period: "+period+" ms");
			sum = sum + period;
		}
		System.out.println("Average = "+(sum / repetitions)+" ms");
		
		System.out.println("\nPublication delays:");
		sum = 0;
		for (int i=0; i<repetitions; i++) {
			period = endPublish[i] - beginPublish[i];
			//System.out.println("Period: "+period+" ms");
			sum = sum + period;
		}
		System.out.println("Average = "+(sum / repetitions)+" ms");
		
		System.out.println("\nNotification delays:");
		sum = 0;
		for (int i=0; i<repetitions; i++) {
				period = notificationReceived[i] - beginPublish[i];
				//System.out.println("Period: "+period+" ms");
				sum = sum + period;
		}
		System.out.println("Event Throughput Average = "+(sum / repetitions)+" ms");
	}

}

class YanceesPublisher extends Thread {
	YanceesRMIClient yancees = null;
	
	public YanceesPublisher(String hostname) {
		super();
		
		try {
			yancees = new YanceesRMIClient(hostname);
		} catch (YanceesException e) {
		   System.out.println("Error when creating YANCEES client");
			System.out.println(e);
			e.printStackTrace();
		}
		
		
	}
	
	public void run() {
		YanceesEvent event;
		System.out.println("YanceesPerformanceTest: Publishing "+YanceesEventNotificationTest.repetitions+" events...");
		
		for (int i=0; i<YanceesEventNotificationTest.repetitions; i++) {
		
			event = newEvent(i);
					
			try {
				YanceesEventNotificationTest.beginPublish[i] = System.currentTimeMillis();
				yancees.publish(event);
				YanceesEventNotificationTest.endPublish[i] = System.currentTimeMillis();
			} catch (YanceesException e1) {
				e1.printStackTrace();
			}
			
			try {
				sleep(150);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			
		}
		System.out.println("Finalizing YanceesPublisher...");
		
	}
	
	public YanceesEvent newEvent(int counter) {
		YanceesEvent event = new YanceesEvent();
		event.put("user_name", "Roberto");
		event.put("age", 29);
		event.put("counter", counter);
		return event;
	}

	
}

 class YanceesSubscriber extends Thread implements SubscriberInterface {
	 YanceesRMIClient yancees = null;
	 String subscriptionFileName = null;
	 
	public YanceesSubscriber(String hostname, String fileName) {
		super();

		subscriptionFileName = fileName;
		try {
			yancees = new YanceesRMIClient(hostname);
		} catch (YanceesException e) {
		   System.out.println("Error when creating YANCEES client");
			System.out.println(e);
			e.printStackTrace();
		}
		
			
	}
		
	public void run() {
		System.out.println("YanceesSubscriber: subscribing...");

		GenericSubscription subscriptionMsg = null;


		// create the subscription message
		try {
			subscriptionMsg = new GenericSubscription(new File(subscriptionFileName));
		} catch (IOException ex) {
			System.out.println(ex);
		}
		if (subscriptionMsg == null) {
			System.out.println("Problem when reading and creating subscription!");
			System.exit(1);
		}
		
		try {
			YanceesEventNotificationTest.beginSubscribe[0] = System.currentTimeMillis();
			yancees.subscribe(
					subscriptionMsg,
					(SubscriberInterface) this);
			YanceesEventNotificationTest.endSubscribe[0] = System.currentTimeMillis();
		} catch (YanceesException e1) {
			e1.printStackTrace();
		}
		
		System.out.println("YanceesSubscriber: waiting for events...");
		
		while (YanceesEventNotificationTest.notifCounter < YanceesEventNotificationTest.repetitions) {
			try {
				sleep(2000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		System.out.println("Finalizing YanceesSubscriber...");
		try {
			yancees.unsubscribe(this);
		} catch (YanceesException e) {
		
			e.printStackTrace();
		}
		YanceesEventNotificationTest.printStatistics();
		
	}
	
	/* (non-Javadoc)
	 * @see siena.Notifiable#notify(siena.Notification)
	 */
	public void notify(EventInterface arg0)  {
		YanceesEventNotificationTest.notificationReceived[YanceesEventNotificationTest.notifCounter] = System.currentTimeMillis();
		YanceesEventNotificationTest.notifCounter++;
		System.out.println("Got noification "+YanceesEventNotificationTest.notifCounter);

	}

	/* (non-Javadoc)
	 * @see siena.Notifiable#notify(siena.Notification[])
	 */
	public void notify(EventInterface[] arg0)  {
		for (int i=0; i<arg0.length; i++) {
			YanceesEventNotificationTest.notificationReceived[YanceesEventNotificationTest.notifCounter] = System.currentTimeMillis();
			YanceesEventNotificationTest.notifCounter++;
			System.out.println("Got noification "+YanceesEventNotificationTest.notifCounter);
		}
		

	}
	

}



