/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 *
 * ===================================================================
 * The Apache Software License, Version 1.1
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 *
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 package edu.uci.isr.yancees.plugin;

/**
 * <p>Title: Yancees Notification Server</p>
 * <p>Description: Yet ANother Configurable Extensible Event Service</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: School of Information and Computer Science - University of California, Irvine</p>
 * @author Roberto Silveira Silva Filho
 * @version 1.0
 */

import org.w3c.dom.Node;

import edu.uci.isr.yancees.SubscriberInterface;
import edu.uci.isr.yancees.SubscriptionInterface;

/**
 * This is the interface of the plug-ins of the core of the system.
 * A plug-in is a listener of the events generated by other plug-ins.
 */

public interface MOPluginInterface
      extends  PluginInterface, MOPluginListenerInterface {
	
	
	/**
	 * @return a reference of the subscriber that triggered the creation of the
	 * plug-in
	 */
	public SubscriberInterface getSubscriber();
	
	/**
	 * @return a reference to the subscription this plug-in participates
	 */
	public SubscriptionInterface getSubscription();

   /**
    * @return a string separated by '/' that expresses the context, or sequence of
    * tags from the root of a subscription DOM subtree to the subtree that this plug-in
    * represents. For example, if this plug-in will handle the subtree headed by
    * the tag <followed-by> in a subscription
    * like:
    *   <subscription>
    *      <rule>
    *        <followed-by>
    *           <filter>...
    *           <filter>...
    *        </followed-by>
    *      </rule>
    *      <action>
    *         ...
    *      </action>
    *   </subscription>
    *
    * its full context will be 'subscription/rule' because followed-by is inside this
    * path. The context is the path without the tag name.
    *
    * This information will be part of all events produced by this component.
          * This information helps the subscription node to identify the events coming
          * from this component, knowing that that specific branch of his was evaluated
    * and the answer comes in that event.
    */
   public String getFullContext();

   /**
    * @return a string separated by '/' that expresses the path, or sequence of
    * tags from the root of a subscription DOM subtree to the subtree that this plug-in
    * represents. For example, if this plug-in will handle the subtree headed by
    * the tag <followed-by> in a subscription
    * like:
    *   <subscription>
    *      <rule>
    *        <followed-by>
    *           <filter>...
    *           <filter>...
    *        </followed-by>
    *      </rule>
    *      <action>
    *         ...
    *      </action>
    *   </subscription>
    *
    * its full path will be 'subscription/rule/followed-by'
    *
    * This information will be part of all events produced by this component.
          * This information helps the subscription node to identify the events coming
          * from this component, knowing that that specific branch of his was evaluated
    * and the answer comes in that event.
    */
   public String getFullPath();

   /**
    * @return the DOM subtree that is handled (and headed) by this plug-in
    */
   public Node getSubtree();

   /**
    * This plugin interface implements the observer design pattern, or a simple
    * publish/subscribe. It allows the plug-in structure to be decoupled from the
    * event dispatcher. The use of the event dispatcher would require the definition
    * of a standard subscription language that, in the case of the initial imple-
    * mentation of this prototype, would be to bound to Siena.
    */

   /**
    * Makes this plug-in notify another provided plugin whenever it computers its output
    * The output here is data (events) obtained as a result of a subscription
    */
   public void addListener(MOPluginListenerInterface plugin);

   /**
    * Removes a plug-in registered as listener of this plug-in
    */
   public void removeListener(MOPluginListenerInterface plugin);

   /**
    * This method is used by the subscription parser to inform the plugin of all
    * the IDs of the plug-ins that are supposed to notify this plugin when they
    * are finished with their processing. This method is invoked in the BFS order
    * in which the tags each plugin processes are parsed in the DOM tree.
    *
    * This information is necessary for plug-ins as the sequence detection in which
    * the order of notifications may be important.
    */
   public void addRequiredPlugin(MOPluginInterface plugin);


   /**
    * @return a list of PluginsInterface instances that this plug-in requires.
    */
   public MOPluginInterface[] getRequiredPluginsList();

   /**
    * Determines whether this plug-in requires other plug-ins or not
    * @return true if this plug-in requires other plug-ins or false if not
    */
   public boolean hasChildren();

	/**
	 * Releases all the listeners of this plug-in in order to make it eligible
	 * for garbage collection.
	 */
	public void dispose();


}
