/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 /*
 * Created on Aug 29, 2003
 *
 * <p>Title: Yancees Notification Server</p>
 * <p>Description: Yet ANother Configurable Extensible Event Service</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: School of Information and Computer Science - University of California, Irvine</p>
 * @author Roberto Silveira Silva Filho
 * @version 1.0
 */
package edu.uci.isr.yancees.server.service;

import java.util.HashMap;
import java.util.Set;


/**
 * This class manages the current instances of services, special plug-ins
 * that can be shared by all other plugins.
 */
public class ServiceManager implements ServiceManagerInterface {

	 // Unique instance of this singleton.
   protected static ServiceManager myInstance;
   
   private boolean print = edu.uci.isr.yancees.YanceesProperties.getInstance().PRINT_DEBUG;
	private HashMap serviceTable = new HashMap();
	
   protected ServiceManager() {
      super();
      if (print) {
         System.out.println("Invoking ServiccesRegistry constructor...");
         //plugManager = PluginManager.getInstance();
      }
   }

   /**
    * The only way to access the unique instance of ProtocolManager is by
    * using this access method
    * @return the unique instance of this class.
    */
   public static ServiceManager getInstance() {
      if (myInstance == null) {
         myInstance = new ServiceManager();
      }
      return myInstance;
   }
   
   /**
    * @param name is the name of the service to be queried
    * @return a service identified by the provided name
    */
   public ServiceInterface getServiceByName(String name) {
   	return (ServiceInterface) serviceTable.get(name);
   }
   
   /**
    * Register a service. The name of the service needs to be unique. It is
    * collected using the getName() method in the service interface
    */
   public void registerService(ServiceInterface service) {
   	serviceTable.put(service.getName(), service);
   }

	/**
	 * Unregisters the service
	 * @param serviceName is the name of the service to be unregistered
	 */
	public void unregisterService(String serviceName) {
		serviceTable.remove(serviceName);

	}

	/* (non-Javadoc)
	 * @see yancees.service.ServiceManagerInterface#getAvailableServices()
	 */
	public ServiceInterface[] getAvailableServices() {
		ServiceInterface[] answerSet;
		Set servicesSet = serviceTable.entrySet();
		Object[] servicesList = servicesSet.toArray();
		answerSet = new ServiceInterface[servicesList.length];
		for (int i = 0; i < servicesList.length; i++) {
			answerSet[i] = (ServiceInterface) servicesList[i];
		}
		return answerSet;
	}

	/* (non-Javadoc)
	 * @see yancees.service.ServiceManagerInterface#listAvailableServiceNames()
	 */
	public String[] listAvailableServiceNames() {
		String[] answerSet;
		Set keySet = serviceTable.keySet();
		Object[] keySetList = keySet.toArray();
		answerSet = new String[keySetList.length];
		for (int i = 0; i < keySetList.length; i++) {
			answerSet[i] = (String) keySetList[i];
		}
		return answerSet;
	}

}
